# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.annotation_opt_out import AnnotationOptOut
from openapi_client.models.campaign.batch_error import BatchError
from typing_extensions import Self

class GetAnnotationOptOutResponse(BaseModel):
    """
    GetAnnotationOptOutResponse
    """ # noqa: E501
    justification: Optional[StrictStr] = Field(default=None, alias="Justification")
    annotation_opt_outs: Optional[List[Optional[AnnotationOptOut]]] = Field(default=None, alias="AnnotationOptOuts")
    is_account_opt_out: Optional[StrictBool] = Field(default=None, alias="IsAccountOptOut")
    is_customer_opt_out: Optional[StrictBool] = Field(default=None, alias="IsCustomerOptOut")
    is_customer_opt_out_of_everything: Optional[StrictBool] = Field(default=None, alias="IsCustomerOptOutOfEverything")
    partial_errors: Optional[List[Optional[BatchError]]] = Field(default=None, alias="PartialErrors")
    __properties: ClassVar[List[str]] = ["Justification", "AnnotationOptOuts", "IsAccountOptOut", "IsCustomerOptOut", "IsCustomerOptOutOfEverything", "PartialErrors"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in annotation_opt_outs (list)
        _items = []
        if self.annotation_opt_outs:
            for _item_annotation_opt_outs in self.annotation_opt_outs:
                if _item_annotation_opt_outs:
                    _items.append(_item_annotation_opt_outs.to_dict())
            _dict['AnnotationOptOuts'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in partial_errors (list)
        _items = []
        if self.partial_errors:
            for _item_partial_errors in self.partial_errors:
                if _item_partial_errors:
                    _items.append(_item_partial_errors.to_dict())
            _dict['PartialErrors'] = _items
        # set to None if justification (nullable) is None
        # and model_fields_set contains the field
        if self.justification is None and "justification" in self.model_fields_set:
            _dict['Justification'] = None

        # set to None if annotation_opt_outs (nullable) is None
        # and model_fields_set contains the field
        if self.annotation_opt_outs is None and "annotation_opt_outs" in self.model_fields_set:
            _dict['AnnotationOptOuts'] = None

        # set to None if is_account_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.is_account_opt_out is None and "is_account_opt_out" in self.model_fields_set:
            _dict['IsAccountOptOut'] = None

        # set to None if is_customer_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.is_customer_opt_out is None and "is_customer_opt_out" in self.model_fields_set:
            _dict['IsCustomerOptOut'] = None

        # set to None if is_customer_opt_out_of_everything (nullable) is None
        # and model_fields_set contains the field
        if self.is_customer_opt_out_of_everything is None and "is_customer_opt_out_of_everything" in self.model_fields_set:
            _dict['IsCustomerOptOutOfEverything'] = None

        # set to None if partial_errors (nullable) is None
        # and model_fields_set contains the field
        if self.partial_errors is None and "partial_errors" in self.model_fields_set:
            _dict['PartialErrors'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetAnnotationOptOutResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Justification": obj.get("Justification") if obj.get("Justification") is not None else None,
                        "AnnotationOptOuts": [AnnotationOptOut.from_dict(_item) for _item in obj["AnnotationOptOuts"]] if obj.get("AnnotationOptOuts") is not None else None,
                        "IsAccountOptOut": obj.get("IsAccountOptOut") if obj.get("IsAccountOptOut") is not None else None,
                        "IsCustomerOptOut": obj.get("IsCustomerOptOut") if obj.get("IsCustomerOptOut") is not None else None,
                        "IsCustomerOptOutOfEverything": obj.get("IsCustomerOptOutOfEverything") if obj.get("IsCustomerOptOutOfEverything") is not None else None,
                        "PartialErrors": [BatchError.from_dict(_item) for _item in obj["PartialErrors"]] if obj.get("PartialErrors") is not None else None
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities import QualityScoreData
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *
from decimal import Decimal

_DynamicFeedSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('DynamicFeedSetting'))
_TargetSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('TargetSetting'))
_ShoppingSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ShoppingSetting'))
_DsaSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('DynamicSearchAdsSetting'))
_DisclaimerSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('DisclaimerSetting'))
_VerifiedTrackingSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('VerifiedTrackingSetting'))
_PerformanceMaxSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('PerformanceMaxSetting'))
_NewCustomerAcquisitionGoalSetting = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('NewCustomerAcquisitionGoalSetting'))

class BulkCampaign(_SingleRecordBulkEntity):
    """ Represents a campaign that can be read or written in a bulk file.

    This class exposes the :attr:`campaign` property that can be read and written as fields of the
    Campaign record in a bulk file.

    For more information, see Campaign at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, campaign=None):
        super(BulkCampaign, self).__init__()

        self._account_id = account_id
        self._campaign = campaign
        self._quality_score_data = None
        self._performance_data = None
        self._budget_name = None
        self._bid_strategy_name = None
        self._verified_tracking_data = None
        self._destination_channel = None
        self._is_multi_channel_campaign = None
        self._should_serve_on_msan = None

    @property
    def account_id(self):
        """ The identifier of the account that contains the campaign.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id

    @property
    def bid_strategy_name(self):
        """
        The budget name that the campaign associated, only for campaigns that use a shared budget

        Corresponds to 'Budget Name' field in bulk file.
        :rtype: str
        """
        return self._bid_strategy_name

    @bid_strategy_name.setter
    def bid_strategy_name(self, value):
        self._bid_strategy_name = value

    @property
    def verified_tracking_data(self):
        """
        The verified tracking data that the campaign associated, only for audience campaigns

        Corresponds to 'Verified Tracking Setting' field in bulk file.
        :rtype: ArrayOfArrayOfKeyValuePairOfstringstring
        """
        return self._verified_tracking_data

    @verified_tracking_data.setter
    def verified_tracking_data(self, value):
        self._verified_tracking_data = value

    @property
    def budget_name(self):
        """
        The budget name that the campaign associated, only for campaigns that use a shared budget

        Corresponds to 'Budget Name' field in bulk file.
        :rtype: str
        """
        return self._budget_name

    @budget_name.setter
    def budget_name(self, value):
        self._budget_name = value

    @property
    def campaign(self):
        """ Defines a campaign within an account.

        See Campaign at https://docs.microsoft.com/en-us/bingads/campaign-management-service/campaign?view=bingads-13
        """

        return self._campaign

    @campaign.setter
    def campaign(self, campaign):
        self._campaign = campaign

    @property
    def quality_score_data(self):
        """ The quality score data for the campaign.

        :rtype: QualityScoreData
        """

        return self._quality_score_data

    @property
    def destination_channel(self):
        return self._destination_channel

    @destination_channel.setter
    def destination_channel(self, value):
        self._destination_channel = value

    @property
    def is_multi_channel_campaign(self):
        return self._is_multi_channel_campaign

    @is_multi_channel_campaign.setter
    def is_multi_channel_campaign(self, value):
        self._is_multi_channel_campaign = value

    @property
    def should_serve_on_msan(self):
        return self._should_serve_on_msan

    @should_serve_on_msan.setter
    def should_serve_on_msan(self, value):
        self._should_serve_on_msan = value

    def _get_dynamic_feed_setting(self):
        return self._get_setting(_DynamicFeedSetting, 'DynamicFeedSetting')

    def _get_shopping_setting(self):
        return self._get_setting(_ShoppingSetting, 'ShoppingSetting')

    def _get_target_setting(self):
        return self._get_setting(_TargetSetting, 'TargetSetting')

    def _get_dsa_setting(self):
        return self._get_setting(_DsaSetting, 'DynamicSearchAdsSetting')

    def _get_disclaimer_setting(self):
        return self._get_setting(_DisclaimerSetting, 'DisclaimerSetting')

    def _get_verified_tracking_setting(self):
        return self._get_setting(_VerifiedTrackingSetting, 'VerifiedTrackingSetting')

    def _get_performance_max_setting(self):
        return self._get_setting(_PerformanceMaxSetting, 'PerformanceMaxSetting')

    def _get_new_customer_acquisition_goal_setting(self):
        return self._get_setting(_NewCustomerAcquisitionGoalSetting, 'NewCustomerAcquisitionGoalSetting')

    def _get_setting(self, setting_type, setting_name):
        if not self.campaign.Settings.Setting:
            return None
        settings = [setting for setting in self.campaign.Settings.Setting if
                             isinstance(setting, setting_type)]
        if len(settings) > 1:
            raise ValueError('Can only have 1 ' + setting_name +  ' at most in Campaign Settings.')
        elif len(settings) == 0:
            return None
        return settings[0]

    @staticmethod
    def _write_campaign_type(c):
        if not c.campaign.CampaignType:
            return None
        if len(c.campaign.CampaignType) != 1:
            raise ValueError("Only 1 CampaignType can be set in Campaign")
        return c.campaign.CampaignType[0]

    @staticmethod
    def _read_campaign_type(c, v):
        if not v:
            return []
        campaign_type = v
        c.campaign.CampaignType = [campaign_type]

        if campaign_type.lower() == 'shopping':
            BulkCampaign._create_campaign_setting(c.campaign, 'ShoppingSetting')
        if campaign_type.lower() == 'audience':
            BulkCampaign._create_campaign_setting(c.campaign, 'DynamicFeedSetting')
            BulkCampaign._create_campaign_setting(c.campaign, 'ShoppingSetting')
            BulkCampaign._create_campaign_setting(c.campaign, 'VerifiedTrackingSetting')
        if campaign_type.lower() == 'dynamicsearchads' or campaign_type.lower() == 'search':
            BulkCampaign._create_campaign_setting(c.campaign, 'DynamicSearchAdsSetting')
            BulkCampaign._create_campaign_setting(c.campaign, 'DisclaimerSetting')
        if campaign_type.lower() == 'performancemax':
            BulkCampaign._create_campaign_setting(c.campaign, 'PerformanceMaxSetting')
            BulkCampaign._create_campaign_setting(c.campaign, 'ShoppingSetting')
            BulkCampaign._create_campaign_setting(c.campaign, 'NewCustomerAcquisitionGoalSetting')

    @staticmethod
    def _create_campaign_setting(campaign, setting_type):
        if not campaign.Settings:
            campaign.Settings = _CAMPAIGN_OBJECT_FACTORY_V13.create('ArrayOfSetting')

        setting = _CAMPAIGN_OBJECT_FACTORY_V13.create(setting_type)
        setting.Type = setting_type
        campaign.Settings.Setting.append(setting)

    @staticmethod
    def _write_final_url_expansion_opt_out(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            return bulk_str(performance_max_setting.FinalUrlExpansionOptOut)

    @staticmethod
    def _read_final_url_expansion_opt_out(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            performance_max_setting.FinalUrlExpansionOptOut = parse_bool(v)

    @staticmethod
    def _write_store_id(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            return bulk_str(shopping_setting.StoreId)

    @staticmethod
    def _read_store_id(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            shopping_setting.StoreId = int(v) if v else None

    @staticmethod
    def _write_priority(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            return bulk_str(shopping_setting.Priority)

    @staticmethod
    def _read_priority(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            shopping_setting.Priority = int(v) if v else None

    @staticmethod
    def _write_sales_country_code(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            return shopping_setting.SalesCountryCode

    @staticmethod
    def _read_sales_country_code(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            shopping_setting.SalesCountryCode = v

    @staticmethod
    def _write_local_inventory_ads_enabled(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            return shopping_setting.LocalInventoryAdsEnabled

    @staticmethod
    def _read_local_inventory_ads_enabled(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'shopping' in campgaign_types or 'audience' in campgaign_types:
            shopping_setting = c._get_shopping_setting()
            if not shopping_setting:
                return None
            shopping_setting.LocalInventoryAdsEnabled = v.lower() == 'true' if v else None

    @staticmethod
    def _write_feed_id(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'audience' in campgaign_types:
            dynamic_feed_setting = c._get_dynamic_feed_setting()
            if not dynamic_feed_setting:
                return None
            return bulk_str(dynamic_feed_setting.FeedId)

    @staticmethod
    def _read_feed_id(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'audience' in campgaign_types:
            dynamic_feed_setting = c._get_dynamic_feed_setting()
            if not dynamic_feed_setting:
                return None
            dynamic_feed_setting.FeedId = int(v) if v else None

    @staticmethod
    def _read_source(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        dsa_setting.Source = v

    @staticmethod
    def _write_source(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        return bulk_str(dsa_setting.Source)

    @staticmethod
    def _read_domain_language(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        dsa_setting.Language = v

    @staticmethod
    def _write_domain_language(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        return bulk_str(dsa_setting.Language)

    @staticmethod
    def _read_DisclaimerAdsEnabled(c, v):
        if not c.campaign.CampaignType:
            return None
        disclaimer_setting = c._get_disclaimer_setting()
        if not disclaimer_setting:
            return None
        disclaimer_setting.DisclaimerAdsEnabled = parse_bool(v)

    @staticmethod
    def _write_DisclaimerAdsEnabled(c):
        if not c.campaign.CampaignType:
            return None
        disclaimer_setting = c._get_disclaimer_setting()
        if not disclaimer_setting:
            return None
        return bulk_str(disclaimer_setting.DisclaimerAdsEnabled)

    @staticmethod
    def _read_DynamicDescriptionEnabled(c, v):
        if not c.campaign.CampaignType:
            return None
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        dsa_setting.DynamicDescriptionEnabled = parse_bool(v)

    @staticmethod
    def _write_DynamicDescriptionEnabled(c):
        if not c.campaign.CampaignType:
            return None
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        return bulk_str(dsa_setting.DynamicDescriptionEnabled)

    @staticmethod
    def _read_page_feed_ids(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            performance_max_setting.PageFeedIds.long = csv_to_field_PageFeedIds(v)
        else:
            dsa_setting = c._get_dsa_setting()
            if not dsa_setting:
                return None
            dsa_setting.PageFeedIds.long = csv_to_field_PageFeedIds(v)

    @staticmethod
    def _write_page_feed_ids(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            return field_to_csv_Ids(performance_max_setting.PageFeedIds, c.campaign.Id)
        else:
            dsa_setting = c._get_dsa_setting()
            if not dsa_setting:
                return None
            return field_to_csv_Ids(dsa_setting.PageFeedIds, c.campaign.Id)

    @staticmethod
    def _read_text_opt_out(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            performance_max_setting.AutoGeneratedTextOptOut = parse_bool(v)

    @staticmethod
    def _write_text_opt_out(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            return bulk_str(performance_max_setting.AutoGeneratedTextOptOut)

    @staticmethod
    def _read_cost_per_sale_opt_out(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            performance_max_setting.CostPerSaleOptOut = parse_bool(v)

    @staticmethod
    def _write_cost_per_sale_opt_out(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            return bulk_str(performance_max_setting.CostPerSaleOptOut)

    @staticmethod
    def _read_image_opt_out(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            performance_max_setting.AutoGeneratedImageOptOut = parse_bool(v)

    @staticmethod
    def _write_image_opt_out(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            performance_max_setting = c._get_performance_max_setting()
            if not performance_max_setting:
                return None
            return bulk_str(performance_max_setting.AutoGeneratedImageOptOut)

    @staticmethod
    def _read_new_customer_acquisition_bid_only_mode(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            new_customer_acquisition_goal_setting.NewCustomerAcquisitionBidOnlyMode = parse_bool(v)

    @staticmethod
    def _write_new_customer_acquisition_bid_only_mode(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            return bulk_str(new_customer_acquisition_goal_setting.NewCustomerAcquisitionBidOnlyMode)

    @staticmethod
    def _read_new_customer_acquisition_goal_id(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            new_customer_acquisition_goal_setting.NewCustomerAcquisitionGoalId = int(v) if v else None

    @staticmethod
    def _write_new_customer_acquisition_goal_id(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            return bulk_str(new_customer_acquisition_goal_setting.NewCustomerAcquisitionGoalId)

    @staticmethod
    def _read_additional_conversion_value(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            new_customer_acquisition_goal_setting.AdditionalConversionValue = Decimal(v) if v else None

    @staticmethod
    def _write_additional_conversion_value(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        if 'performancemax' in campgaign_types:
            new_customer_acquisition_goal_setting = c._get_new_customer_acquisition_goal_setting()
            if not new_customer_acquisition_goal_setting:
                return None
            return bulk_str(new_customer_acquisition_goal_setting.AdditionalConversionValue)

    @staticmethod
    def _read_website(c, v):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        dsa_setting.DomainName = v

    @staticmethod
    def _write_website(c):
        if not c.campaign.CampaignType:
            return None
        campgaign_types = [campaign_type.lower() for campaign_type in c.campaign.CampaignType]
        dsa_setting = c._get_dsa_setting()
        if not dsa_setting:
            return None
        return bulk_str(dsa_setting.DomainName)

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.CampaignType,
            field_to_csv=lambda c: BulkCampaign._write_campaign_type(c),
            csv_to_field=lambda c, v: BulkCampaign._read_campaign_type(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.campaign.Status),
            csv_to_field=lambda c, v: csv_to_field_enum(c.campaign, v, 'Status', CampaignStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.campaign.Id),
            csv_to_field=lambda c, v: setattr(
                c.campaign,
                'Id',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, '_account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign.Name,
            csv_to_field=lambda c, v: setattr(c.campaign, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TimeZone,
            field_to_csv=lambda c: c.campaign.TimeZone,
            csv_to_field=lambda c, v: setattr(c.campaign, 'TimeZone', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Language,
            field_to_csv=lambda c: field_to_csv_CampaignLanguages(c.campaign.Languages),
            csv_to_field=lambda c, v: csv_to_field_CampaignLanguages(c.campaign.Languages, v)
        ),
        _ComplexBulkMapping(budget_to_csv, csv_to_budget),
        _SimpleBulkMapping(
            header=_StringTable.BidAdjustment,
            field_to_csv=lambda c: bulk_str(c.campaign.AudienceAdsBidAdjustment),
            csv_to_field=lambda c, v: setattr(
                c.campaign,
                'AudienceAdsBidAdjustment',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlExpansionOptOut,
            field_to_csv=lambda c: BulkCampaign._write_final_url_expansion_opt_out(c),
            csv_to_field=lambda c, v: BulkCampaign._read_final_url_expansion_opt_out(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MerchantCenterId,
            field_to_csv=lambda c: BulkCampaign._write_store_id(c),
            csv_to_field=lambda c, v: BulkCampaign._read_store_id(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CampaignPriority,
            field_to_csv=lambda c: BulkCampaign._write_priority(c),
            csv_to_field=lambda c, v: BulkCampaign._read_priority(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CountryCode,
            field_to_csv=lambda c: BulkCampaign._write_sales_country_code(c),
            csv_to_field=lambda c, v: BulkCampaign._read_sales_country_code(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LocalInventoryAdsEnabled,
            field_to_csv=lambda c: BulkCampaign._write_local_inventory_ads_enabled(c),
            csv_to_field=lambda c, v: BulkCampaign._read_local_inventory_ads_enabled(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_optional_str(c.campaign.TrackingUrlTemplate, c.campaign.Id),
            csv_to_field=lambda c, v: setattr(c.campaign, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.campaign),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.campaign, v)
        ),
        _ComplexBulkMapping(campaign_biddingscheme_to_csv, csv_to_campaign_biddingscheme),
        _SimpleBulkMapping(
            header=_StringTable.BudgetId,
            field_to_csv=lambda c: bulk_str(c.campaign.BudgetId),
            csv_to_field=lambda c, v: setattr(c.campaign, 'BudgetId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BudgetType,
            field_to_csv=lambda c: bulk_str(c.campaign.BudgetType),
            csv_to_field=lambda c, v: csv_to_field_BudgetType(c.campaign, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BudgetName,
            field_to_csv=lambda c: bulk_str(c.budget_name),
            csv_to_field=lambda c, v: setattr(c, 'budget_name', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BidStrategyName,
            field_to_csv=lambda c: bulk_str(c.bid_strategy_name),
            csv_to_field=lambda c, v: setattr(c, 'bid_strategy_name', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BidStrategyId,
            field_to_csv=lambda c: bulk_str(c.campaign.BidStrategyId),
            csv_to_field=lambda c, v: setattr(c.campaign, 'BidStrategyId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Website,
            field_to_csv=lambda c: BulkCampaign._write_website(c),
            csv_to_field=lambda c, v: BulkCampaign._read_website(c, v)

        ),
        _SimpleBulkMapping(
            header=_StringTable.DomainLanguage,
            field_to_csv=lambda c: BulkCampaign._write_domain_language(c),
            csv_to_field=lambda c, v: BulkCampaign._read_domain_language(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Source,
            field_to_csv=lambda c: BulkCampaign._write_source(c),
            csv_to_field=lambda c, v: BulkCampaign._read_source(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SubType,
            field_to_csv=lambda c: c.campaign.SubType,
            csv_to_field=lambda c, v: setattr(c.campaign, 'SubType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ExperimentId,
            field_to_csv=lambda c: bulk_str(c.campaign.ExperimentId),
            csv_to_field=lambda c, v: setattr(c.campaign, 'ExperimentId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FeedId,
            field_to_csv=lambda c: BulkCampaign._write_feed_id(c),
            csv_to_field=lambda c, v: BulkCampaign._read_feed_id(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TargetSetting,
            field_to_csv=lambda c: target_setting_to_csv(c.campaign),
            csv_to_field=lambda c, v: csv_to_target_setting(c.campaign, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.campaign.FinalUrlSuffix, c.campaign.Id),
            csv_to_field=lambda c, v: setattr(c.campaign, 'FinalUrlSuffix', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PageFeedIds,
            field_to_csv=lambda c: BulkCampaign._write_page_feed_ids(c),
            csv_to_field=lambda c, v: BulkCampaign._read_page_feed_ids(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdScheduleUseSearcherTimeZone,
            field_to_csv=lambda c: field_to_csv_UseSearcherTimeZone(c.campaign.AdScheduleUseSearcherTimeZone, None),
            csv_to_field=lambda c, v: setattr(c.campaign, 'AdScheduleUseSearcherTimeZone', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.MultiMediaAdBidAdjustment,
            field_to_csv=lambda c: bulk_str(c.campaign.MultimediaAdsBidAdjustment),
            csv_to_field=lambda c, v: setattr(
                c.campaign,
                'MultimediaAdsBidAdjustment',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.DisclaimerAdsEnabled,
            field_to_csv=lambda c: BulkCampaign._write_DisclaimerAdsEnabled(c),
            csv_to_field=lambda c, v: BulkCampaign._read_DisclaimerAdsEnabled(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DynamicDescriptionEnabled,
            field_to_csv=lambda c: BulkCampaign._write_DynamicDescriptionEnabled(c),
            csv_to_field=lambda c, v: BulkCampaign._read_DynamicDescriptionEnabled(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DestinationChannel,
            field_to_csv=lambda c: c.destination_channel,
            csv_to_field=lambda c, v: setattr(c, 'destination_channel', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IsMultiChannelCampaign,
            field_to_csv=lambda c: field_to_csv_bool(c.is_multi_channel_campaign),
            csv_to_field=lambda c, v: setattr(c, 'is_multi_channel_campaign', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AutoGeneratedTextOptOut,
            field_to_csv=lambda c: BulkCampaign._write_text_opt_out(c),
            csv_to_field=lambda c, v: BulkCampaign._read_text_opt_out(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AutoGeneratedImageOptOut,
            field_to_csv=lambda c: BulkCampaign._write_image_opt_out(c),
            csv_to_field=lambda c, v: BulkCampaign._read_image_opt_out(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CostPerSaleOptOut,
            field_to_csv=lambda c: BulkCampaign._write_cost_per_sale_opt_out(c),
            csv_to_field=lambda c, v: BulkCampaign._read_cost_per_sale_opt_out(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ShouldServeOnMSAN,
            field_to_csv=lambda c: field_to_csv_bool(c.should_serve_on_msan),
            csv_to_field=lambda c, v: setattr(c, 'should_serve_on_msan', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.NewCustomerAcquisitionGoalId,
            field_to_csv=lambda c: BulkCampaign._write_new_customer_acquisition_goal_id(c),
            csv_to_field=lambda c, v: BulkCampaign._read_new_customer_acquisition_goal_id(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.NewCustomerAcquisitionBidOnlyMode,
            field_to_csv=lambda c: BulkCampaign._write_new_customer_acquisition_bid_only_mode(c),
            csv_to_field=lambda c, v: BulkCampaign._read_new_customer_acquisition_bid_only_mode(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdditionalConversionValue,
            field_to_csv=lambda c: BulkCampaign._write_additional_conversion_value(c),
            csv_to_field=lambda c, v: BulkCampaign._read_additional_conversion_value(c, v)
        ),
    ]

    def read_additional_data(self, stream_reader):
        super(BulkCampaign, self).read_additional_data(stream_reader)

    def process_mappings_from_row_values(self, row_values):
        self._campaign = _CAMPAIGN_OBJECT_FACTORY_V13.create('Campaign')
        row_values.convert_to_entity(self, BulkCampaign._MAPPINGS)
        self._quality_score_data = QualityScoreData.read_from_row_values_or_null(row_values)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.campaign, 'campaign')
        self.convert_to_values(row_values, BulkCampaign._MAPPINGS)
        if not exclude_readonly_data:
            QualityScoreData.write_to_row_values_if_not_null(self.quality_score_data, row_values)



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.reporting.account_status_report_filter import AccountStatusReportFilter
from openapi_client.models.reporting.ad_distribution_report_filter import AdDistributionReportFilter
from openapi_client.models.reporting.ad_group_status_report_filter import AdGroupStatusReportFilter
from openapi_client.models.reporting.ad_type_report_filter import AdTypeReportFilter
from openapi_client.models.reporting.bid_match_type_report_filter import BidMatchTypeReportFilter
from openapi_client.models.reporting.bid_strategy_type_report_filter import BidStrategyTypeReportFilter
from openapi_client.models.reporting.campaign_status_report_filter import CampaignStatusReportFilter
from openapi_client.models.reporting.delivered_match_type_report_filter import DeliveredMatchTypeReportFilter
from openapi_client.models.reporting.device_type_report_filter import DeviceTypeReportFilter
from openapi_client.models.reporting.keyword_status_report_filter import KeywordStatusReportFilter
from openapi_client.models.reporting.language_report_filter import LanguageReportFilter
from typing_extensions import Self

class KeywordPerformanceReportFilter(BaseModel):
    """
    KeywordPerformanceReportFilter
    """ # noqa: E501
    ad_distribution: Optional[AdDistributionReportFilter] = Field(default=None, alias="AdDistribution")
    delivered_match_type: Optional[DeliveredMatchTypeReportFilter] = Field(default=None, alias="DeliveredMatchType")
    keywords: Optional[List[StrictStr]] = Field(default=None, alias="Keywords")
    ad_type: Optional[AdTypeReportFilter] = Field(default=None, alias="AdType")
    bid_match_type: Optional[BidMatchTypeReportFilter] = Field(default=None, alias="BidMatchType")
    device_type: Optional[DeviceTypeReportFilter] = Field(default=None, alias="DeviceType")
    language: Optional[LanguageReportFilter] = Field(default=None, alias="Language")
    quality_score: Optional[List[StrictInt]] = Field(default=None, alias="QualityScore")
    expected_ctr: Optional[List[StrictInt]] = Field(default=None, alias="ExpectedCtr")
    ad_relevance: Optional[List[StrictInt]] = Field(default=None, alias="AdRelevance")
    landing_page_experience: Optional[List[StrictInt]] = Field(default=None, alias="LandingPageExperience")
    account_status: Optional[AccountStatusReportFilter] = Field(default=None, alias="AccountStatus")
    campaign_status: Optional[CampaignStatusReportFilter] = Field(default=None, alias="CampaignStatus")
    ad_group_status: Optional[AdGroupStatusReportFilter] = Field(default=None, alias="AdGroupStatus")
    keyword_status: Optional[KeywordStatusReportFilter] = Field(default=None, alias="KeywordStatus")
    bid_strategy_type: Optional[BidStrategyTypeReportFilter] = Field(default=None, alias="BidStrategyType")
    __properties: ClassVar[List[str]] = ["AdDistribution", "DeliveredMatchType", "Keywords", "AdType", "BidMatchType", "DeviceType", "Language", "QualityScore", "ExpectedCtr", "AdRelevance", "LandingPageExperience", "AccountStatus", "CampaignStatus", "AdGroupStatus", "KeywordStatus", "BidStrategyType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if ad_distribution (nullable) is None
        # and model_fields_set contains the field
        if self.ad_distribution is None and "ad_distribution" in self.model_fields_set:
            _dict['AdDistribution'] = None

        # set to None if delivered_match_type (nullable) is None
        # and model_fields_set contains the field
        if self.delivered_match_type is None and "delivered_match_type" in self.model_fields_set:
            _dict['DeliveredMatchType'] = None

        # set to None if keywords (nullable) is None
        # and model_fields_set contains the field
        if self.keywords is None and "keywords" in self.model_fields_set:
            _dict['Keywords'] = None

        # set to None if ad_type (nullable) is None
        # and model_fields_set contains the field
        if self.ad_type is None and "ad_type" in self.model_fields_set:
            _dict['AdType'] = None

        # set to None if bid_match_type (nullable) is None
        # and model_fields_set contains the field
        if self.bid_match_type is None and "bid_match_type" in self.model_fields_set:
            _dict['BidMatchType'] = None

        # set to None if device_type (nullable) is None
        # and model_fields_set contains the field
        if self.device_type is None and "device_type" in self.model_fields_set:
            _dict['DeviceType'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if quality_score (nullable) is None
        # and model_fields_set contains the field
        if self.quality_score is None and "quality_score" in self.model_fields_set:
            _dict['QualityScore'] = None

        # set to None if expected_ctr (nullable) is None
        # and model_fields_set contains the field
        if self.expected_ctr is None and "expected_ctr" in self.model_fields_set:
            _dict['ExpectedCtr'] = None

        # set to None if ad_relevance (nullable) is None
        # and model_fields_set contains the field
        if self.ad_relevance is None and "ad_relevance" in self.model_fields_set:
            _dict['AdRelevance'] = None

        # set to None if landing_page_experience (nullable) is None
        # and model_fields_set contains the field
        if self.landing_page_experience is None and "landing_page_experience" in self.model_fields_set:
            _dict['LandingPageExperience'] = None

        # set to None if account_status (nullable) is None
        # and model_fields_set contains the field
        if self.account_status is None and "account_status" in self.model_fields_set:
            _dict['AccountStatus'] = None

        # set to None if campaign_status (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_status is None and "campaign_status" in self.model_fields_set:
            _dict['CampaignStatus'] = None

        # set to None if ad_group_status (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_status is None and "ad_group_status" in self.model_fields_set:
            _dict['AdGroupStatus'] = None

        # set to None if keyword_status (nullable) is None
        # and model_fields_set contains the field
        if self.keyword_status is None and "keyword_status" in self.model_fields_set:
            _dict['KeywordStatus'] = None

        # set to None if bid_strategy_type (nullable) is None
        # and model_fields_set contains the field
        if self.bid_strategy_type is None and "bid_strategy_type" in self.model_fields_set:
            _dict['BidStrategyType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of KeywordPerformanceReportFilter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "AdDistribution": obj.get("AdDistribution") if obj.get("AdDistribution") is not None else None,
                        "DeliveredMatchType": obj.get("DeliveredMatchType") if obj.get("DeliveredMatchType") is not None else None,
                        "Keywords": obj.get("Keywords"),
                        "AdType": obj.get("AdType") if obj.get("AdType") is not None else None,
                        "BidMatchType": obj.get("BidMatchType") if obj.get("BidMatchType") is not None else None,
                        "DeviceType": obj.get("DeviceType") if obj.get("DeviceType") is not None else None,
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "QualityScore": obj.get("QualityScore"),
                        "ExpectedCtr": obj.get("ExpectedCtr"),
                        "AdRelevance": obj.get("AdRelevance"),
                        "LandingPageExperience": obj.get("LandingPageExperience"),
                        "AccountStatus": obj.get("AccountStatus") if obj.get("AccountStatus") is not None else None,
                        "CampaignStatus": obj.get("CampaignStatus") if obj.get("CampaignStatus") is not None else None,
                        "AdGroupStatus": obj.get("AdGroupStatus") if obj.get("AdGroupStatus") is not None else None,
                        "KeywordStatus": obj.get("KeywordStatus") if obj.get("KeywordStatus") is not None else None,
                        "BidStrategyType": obj.get("BidStrategyType") if obj.get("BidStrategyType") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.key_value_pair_ofstring_and_byte_array import KeyValuePairOfstringAndByteArray
from openapi_client.models.customer.tax_certificate_status import TaxCertificateStatus
from typing_extensions import Self

class AccountTaxCertificate(BaseModel):
    """
    AccountTaxCertificate
    """ # noqa: E501
    tax_certificate_blob_container_name: Optional[StrictStr] = Field(default=None, alias="TaxCertificateBlobContainerName")
    tax_certificates: Optional[List[Optional[KeyValuePairOfstringAndByteArray]]] = Field(default=None, alias="TaxCertificates")
    status: Optional[TaxCertificateStatus] = Field(default=None, alias="Status")
    __properties: ClassVar[List[str]] = ["TaxCertificateBlobContainerName", "TaxCertificates", "Status"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in tax_certificates (list)
        _items = []
        if self.tax_certificates:
            for _item_tax_certificates in self.tax_certificates:
                if _item_tax_certificates:
                    _items.append(_item_tax_certificates.to_dict())
            _dict['TaxCertificates'] = _items
        # set to None if tax_certificate_blob_container_name (nullable) is None
        # and model_fields_set contains the field
        if self.tax_certificate_blob_container_name is None and "tax_certificate_blob_container_name" in self.model_fields_set:
            _dict['TaxCertificateBlobContainerName'] = None

        # set to None if tax_certificates (nullable) is None
        # and model_fields_set contains the field
        if self.tax_certificates is None and "tax_certificates" in self.model_fields_set:
            _dict['TaxCertificates'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AccountTaxCertificate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "TaxCertificateBlobContainerName": obj.get("TaxCertificateBlobContainerName") if obj.get("TaxCertificateBlobContainerName") is not None else None,
                        "TaxCertificates": [KeyValuePairOfstringAndByteArray.from_dict(_item) for _item in obj["TaxCertificates"]] if obj.get("TaxCertificates") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None
        })
        return _obj



import time
import contextlib
import ssl
import requests
import zipfile
import os
import sys
import shutil

from .bulk_operation_status import *
from .bulk_operation_progress_info import *
from .exceptions import *
from bingads.util import _PollingBlocker, errorcode_of_exception, ratelimit_retry_duration, \
    operation_errorcode_of_exception
from bingads.exceptions import *

from bingads.service_client import ServiceClient
from bingads.manifest import *

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager


class TlsHttpAdapter(HTTPAdapter):
    """" Transport adapter that chooses the TLS protocols based on python versions. """

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            block=block,
            ssl_version=ssl.PROTOCOL_SSLv23
        )


class BulkOperation(object):
    """ The base class that can be derived to represent a bulk operation requested by a user.

    You can use either the :class:`.BulkDownloadOperation` or :class:`.BulkUploadOperation`
    derived class to poll for the operation status, and then download the results file when available.
    """

    def __init__(self,
                 request_id,
                 authorization_data,
                 poll_interval_in_milliseconds=5000,
                 environment='production',
                 tracking_id=None,
                 location=None,
                 **suds_options):
        self._request_id = request_id
        self._service_client = ServiceClient('Bulk', 13, authorization_data, environment, location, **suds_options)
        self._authorization_data = authorization_data
        self._poll_interval_in_milliseconds = poll_interval_in_milliseconds
        self._final_status = None
        self.tracking_id = tracking_id

    def download_result_file(self, result_file_directory, result_file_name, decompress, overwrite,
                             timeout_in_milliseconds=None):
        """ Download file with specified URL and download parameters.

        :param result_file_directory: The download result local directory name.
        :type result_file_directory: str
        :param result_file_name: The download result local file name.
        :type result_file_name: str | None
        :param decompress: Determines whether to decompress the ZIP file.
                            If set to true, the file will be decompressed after download.
                            The default value is false, in which case the downloaded file is not decompressed.
        :type decompress: bool
        :param overwrite: Indicates whether the result file should overwrite the existing file if any.
        :type overwrite: bool
        :param timeout_in_milliseconds: (optional) timeout for download result file in milliseconds
        :type timeout_in_milliseconds: int
        :return: The download file path.
        :rtype: str
        """

        if result_file_directory is None:
            raise ValueError('result_file_directory cannot be None.')

        url = self.final_status.result_file_url
        if result_file_name is None:
            result_file_name = self.request_id

        if decompress:
            name, ext = os.path.splitext(result_file_name)
            if ext == '.zip':
                raise ValueError("Result file can't be decompressed into a file with extension 'zip'."
                                 " Please change the extension of the result_file_name or pass decompress_result_file = false")
            zip_file_path = os.path.join(result_file_directory, name + '.zip')
            result_file_path = os.path.join(result_file_directory, result_file_name)
        else:
            result_file_path = os.path.join(result_file_directory, result_file_name)
            zip_file_path = result_file_path

        if os.path.exists(result_file_path) and overwrite is False:
            raise FileExistsError('Result file: {0} exists'.format(result_file_path))

        headers = {
            'User-Agent': USER_AGENT,
        }
        s = requests.Session()
        s.mount('https://', TlsHttpAdapter())
        timeout_seconds = None if timeout_in_milliseconds is None else timeout_in_milliseconds / 1000.0
        try:
            r = s.get(url, headers=headers, stream=True, verify=True, timeout=timeout_seconds)
        except requests.Timeout as ex:
            raise FileDownloadException(ex)
        r.raise_for_status()
        try:
            with open(zip_file_path, 'wb') as f:
                for chunk in r.iter_content(chunk_size=4096):
                    if chunk:
                        f.write(chunk)
                        f.flush()
            if decompress:
                with contextlib.closing(zipfile.ZipFile(zip_file_path)) as compressed:
                    first = compressed.namelist()[0]
                    with open(result_file_path, 'wb') as f, compressed.open(first, 'r') as cc:
                        # f.write(compressed.read(first))
                        shutil.copyfileobj(cc, f)
        except Exception as ex:
            raise ex
        finally:
            if decompress and os.path.exists(zip_file_path):
                os.remove(zip_file_path)
        return result_file_path

    def track(self, percent_complete=None):
        raise NotImplementedError()

    def get_status(self):
        raise NotImplementedError()

    @property
    def request_id(self):
        """ The request identifier corresponding to the bulk upload or download, depending on the derived type.

        :rtype: str
        """

        return self._request_id

    @property
    def final_status(self):
        """ Gets the final status of the bulk operation or null if the operation is still running.

        :rtype: BulkOperationStatus
        """

        return self._final_status

    @property
    def service_client(self):
        """ The internal bulk service client.

        :rtype: ServiceClient
        """

        return self._service_client

    @property
    def poll_interval_in_milliseconds(self):
        """ The time interval in milliseconds between two status polling attempts.

        :rtype: int
        """

        return self._poll_interval_in_milliseconds

    @poll_interval_in_milliseconds.setter
    def poll_interval_in_milliseconds(self, poll_interval):
        self._poll_interval_in_milliseconds = poll_interval


class BulkDownloadOperation(BulkOperation):
    """ Represents a bulk download operation requested by a user.

    You can use this class to poll for the download status, and then download the file when available.

    *Example:*

    The :meth:`.BulkServiceManager.submit_download` method returns an instance of this class.
    If for any reason you do not want to wait for the file to be prepared for download,
    for example if your application quits unexpectedly or you have other tasks to process, you can
    use an instance of :class:`.BulkDownloadOperation` to download the file when it is available.
    """

    def __init__(self,
                 request_id,
                 authorization_data,
                 poll_interval_in_milliseconds=5000,
                 environment='production',
                 tracking_id=None,
                 location=None,
                 **suds_options):
        super(BulkDownloadOperation, self).__init__(
            request_id=request_id,
            authorization_data=authorization_data,
            poll_interval_in_milliseconds=poll_interval_in_milliseconds,
            environment=environment,
            tracking_id=tracking_id,
            location=location,
            **suds_options
        )

    def track(self, progress=None, timeout_in_milliseconds=None):
        """ Runs until the bulk service has finished processing the download or upload request.

        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: The final BulkOperationStatus.
        :rtype: BulkOperationStatus
        """

        if self.final_status is not None:
            return self.final_status
        blocker = _PollingBlocker(self.poll_interval_in_milliseconds, timeout_in_milliseconds)
        blocker.wait()
        while True:
            status = self.get_status()
            percentage = int(status.percent_complete)
            if progress is not None:
                progress(BulkOperationProgressInfo(percentage))
            if status.status == 'InProgress':
                blocker.wait()
                continue
            if status.status != 'Completed':
                raise BulkException('Exceptions while bulk download.', status.errors)
            self._final_status = status
            return self._final_status

    def get_status(self):
        """ Track the detailed download status.

        :return: The status of bulk download operation.
        :rtype: BulkOperationStatus
        """

        if self.final_status is not None:
            return self.final_status
        response = self._get_status_with_retry(4)
        headers = self.service_client.get_response_header()
        self.tracking_id = headers['TrackingId'] if 'TrackingId' in headers else None
        status = BulkOperationStatus(
            status=response.RequestStatus,
            percent_complete=int(response.PercentComplete),
            result_file_url=response.ResultFileUrl,
            errors=None if response.Errors is None else [
                OperationError(
                    code=error.Code,
                    details=error.Details,
                    error_code=error.ErrorCode,
                    message=error.Message,
                ) for error in response.Errors.OperationError
            ]
        )
        if status.status == 'Completed' or \
                status.status == 'Failed' or \
                status.status == 'FailedFullSyncRequired':
            self._final_status = status
        return status

    def _get_status_with_retry(self, retry_times):
        from openapi_client.models.bulk import GetBulkDownloadStatusRequest

        request_id = (str(self.request_id)).split("'")[1]
        while retry_times > 1:
            try:
                request = GetBulkDownloadStatusRequest(
                    request_id=request_id
                )
                return self.service_client.get_bulk_download_status(
                    get_bulk_download_status_request=request
                )
            except Exception as ex:
                retry_times -= 1
                if '117' == errorcode_of_exception(ex):
                    time.sleep(ratelimit_retry_duration[3 - retry_times])
                else:
                    time.sleep(1)
        request = GetBulkDownloadStatusRequest(
            request_id=request_id
        )
        return self.service_client.get_bulk_download_status(
            get_bulk_download_status_request=request
        )


class BulkUploadOperation(BulkOperation):
    """ Represents a bulk upload operation requested by a user.

    You can use this class to poll for the upload status, and then download the upload results file when available.

    *Example:*

    The :meth:`.BulkServiceManager.submit_upload` method returns an instance of this class.
    If for any reason you do not want to wait for the file to finish uploading,
    for example if your application quits unexpectedly or you have other tasks to process, you can
    use an instance of :class:`.BulkUploadOperation` to download the upload results file when it is available.
    """

    def __init__(self,
                 request_id,
                 authorization_data,
                 poll_interval_in_milliseconds=5000,
                 environment='production',
                 tracking_id=None,
                 location=None,
                 **suds_options):
        super(BulkUploadOperation, self).__init__(
            request_id=request_id,
            authorization_data=authorization_data,
            poll_interval_in_milliseconds=poll_interval_in_milliseconds,
            environment=environment,
            tracking_id=tracking_id,
            location=location
            **suds_options
        )

    def track(self, progress=None, timeout_in_milliseconds=None):
        """ Runs until the bulk service has finished processing the download or upload request.

        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: The final BulkOperationStatus.
        :rtype: BulkOperationStatus
        """

        if self.final_status is not None:
            return self.final_status
        blocker = _PollingBlocker(self.poll_interval_in_milliseconds, timeout_in_milliseconds)
        blocker.wait()
        while True:
            status = self.get_status()
            percentage = int(status.percent_complete)
            if progress is not None:
                progress(BulkOperationProgressInfo(percentage))
            if status.status == 'InProgress' or status.status == 'FileUploaded' or status.status == 'PendingFileUpload':
                blocker.wait()
                continue
            if status.status != 'Completed' and status.status != 'CompletedWithErrors':
                raise BulkException("Exceptions while bulk upload.", status.errors)
            self._final_status = status
            return self._final_status

    def get_status(self):
        """ Track the detailed upload status.

        :return: The status of bulk upload operation.
        :rtype: BulkOperationStatus
        """

        if self.final_status is not None:
            return self.final_status
        response = self._get_status_with_retry(4)
        headers = self.service_client.get_response_header()
        self.tracking_id = headers['TrackingId'] if 'TrackingId' in headers else None
        status = BulkOperationStatus(
            status=response.RequestStatus,
            percent_complete=int(response.PercentComplete),
            result_file_url=response.ResultFileUrl,
            errors=None if response.Errors is None else [
                OperationError(
                    code=error.Code,
                    details=error.Details,
                    error_code=error.ErrorCode,
                    message=error.Message,
                ) for error in response.Errors.OperationError
            ]
        )
        if status.status == 'Completed' or \
                status.status == 'CompletedWithErrors' or \
                status.status == 'Failed' or \
                status.status == 'Expired' or \
                status.status == 'Aborted':
            self._final_status = status
        return status

    def _get_status_with_retry(self, retry_times):
        from openapi_client.models.bulk import GetBulkUploadStatusRequest
        
        while retry_times > 1:
            try:
                request = GetBulkUploadStatusRequest(
                    request_id=self.request_id
                )
                return self.service_client.get_bulk_upload_status(
                    get_bulk_upload_status_request=request
                )
            except Exception as ex:
                retry_times -= 1
                if '117' == errorcode_of_exception(ex):
                    time.sleep(ratelimit_retry_duration[3 - retry_times])
                else:
                    time.sleep(1)
        request = GetBulkUploadStatusRequest(
            request_id=self.request_id
        )
        return self.service_client.get_bulk_upload_status(
            get_bulk_upload_status_request=request
        )



import csv
import io

class _CsvReader:

    def __init__(self, filename, delimiter, encoding='utf-8-sig'):
        self._filename = filename
        self._delimiter = delimiter
        self._encoding = encoding

        if delimiter == ',':
            self._dialect = csv.excel
        elif delimiter == '\t':
            self._dialect = csv.excel_tab
        else:
            raise ValueError('Do not support delimiter: {0}', delimiter)

        self._csv_file = io.open(self.filename, encoding=self.encoding)

        self._csv_reader = csv.reader(self._csv_file, dialect=self._dialect)
        
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._csv_file.__exit__(exc_type, exc_value, traceback)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def close(self):
        self.__exit__(None, None, None)

    def next(self):
        return next(self._csv_reader)
        
    @property
    def filename(self):
        return self._filename

    @property
    def delimiter(self):
        return self._delimiter

    @property
    def encoding(self):
        return self._encoding



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.campaign.bid import Bid
from openapi_client.models.campaign.commission_bidding_scheme import CommissionBiddingScheme
from openapi_client.models.campaign.cost_per_sale_bidding_scheme import CostPerSaleBiddingScheme
from openapi_client.models.campaign.enhanced_cpc_bidding_scheme import EnhancedCpcBiddingScheme
from openapi_client.models.campaign.inherit_from_parent_bidding_scheme import InheritFromParentBiddingScheme
from openapi_client.models.campaign.manual_cpa_bidding_scheme import ManualCpaBiddingScheme
from openapi_client.models.campaign.manual_cpc_bidding_scheme import ManualCpcBiddingScheme
from openapi_client.models.campaign.manual_cpm_bidding_scheme import ManualCpmBiddingScheme
from openapi_client.models.campaign.manual_cpv_bidding_scheme import ManualCpvBiddingScheme
from openapi_client.models.campaign.max_clicks_bidding_scheme import MaxClicksBiddingScheme
from openapi_client.models.campaign.max_conversion_value_bidding_scheme import MaxConversionValueBiddingScheme
from openapi_client.models.campaign.max_conversions_bidding_scheme import MaxConversionsBiddingScheme
from openapi_client.models.campaign.max_roas_bidding_scheme import MaxRoasBiddingScheme
from openapi_client.models.campaign.percent_cpc_bidding_scheme import PercentCpcBiddingScheme
from openapi_client.models.campaign.target_cpa_bidding_scheme import TargetCpaBiddingScheme
from openapi_client.models.campaign.target_impression_share_bidding_scheme import TargetImpressionShareBiddingScheme
from openapi_client.models.campaign.target_roas_bidding_scheme import TargetRoasBiddingScheme
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

BiddingScheme_ONE_OF_SCHEMAS = ["CommissionBiddingScheme", "CostPerSaleBiddingScheme", "EnhancedCpcBiddingScheme", "InheritFromParentBiddingScheme", "ManualCpaBiddingScheme", "ManualCpcBiddingScheme", "ManualCpmBiddingScheme", "ManualCpvBiddingScheme", "MaxClicksBiddingScheme", "MaxConversionValueBiddingScheme", "MaxConversionsBiddingScheme", "MaxRoasBiddingScheme", "PercentCpcBiddingScheme", "TargetCpaBiddingScheme", "TargetImpressionShareBiddingScheme", "TargetRoasBiddingScheme"]

class BiddingScheme(BaseModel):
    """
    BiddingScheme
    """
    # data type: CommissionBiddingScheme
    oneof_schema_commission_bidding_scheme_validator: Optional[CommissionBiddingScheme] = None
    # data type: CostPerSaleBiddingScheme
    oneof_schema_cost_per_sale_bidding_scheme_validator: Optional[CostPerSaleBiddingScheme] = None
    # data type: EnhancedCpcBiddingScheme
    oneof_schema_enhanced_cpc_bidding_scheme_validator: Optional[EnhancedCpcBiddingScheme] = None
    # data type: InheritFromParentBiddingScheme
    oneof_schema_inherit_from_parent_bidding_scheme_validator: Optional[InheritFromParentBiddingScheme] = None
    # data type: ManualCpaBiddingScheme
    oneof_schema_manual_cpa_bidding_scheme_validator: Optional[ManualCpaBiddingScheme] = None
    # data type: ManualCpcBiddingScheme
    oneof_schema_manual_cpc_bidding_scheme_validator: Optional[ManualCpcBiddingScheme] = None
    # data type: ManualCpmBiddingScheme
    oneof_schema_manual_cpm_bidding_scheme_validator: Optional[ManualCpmBiddingScheme] = None
    # data type: ManualCpvBiddingScheme
    oneof_schema_manual_cpv_bidding_scheme_validator: Optional[ManualCpvBiddingScheme] = None
    # data type: MaxClicksBiddingScheme
    oneof_schema_max_clicks_bidding_scheme_validator: Optional[MaxClicksBiddingScheme] = None
    # data type: MaxConversionValueBiddingScheme
    oneof_schema_max_conversion_value_bidding_scheme_validator: Optional[MaxConversionValueBiddingScheme] = None
    # data type: MaxConversionsBiddingScheme
    oneof_schema_max_conversions_bidding_scheme_validator: Optional[MaxConversionsBiddingScheme] = None
    # data type: MaxRoasBiddingScheme
    oneof_schema_max_roas_bidding_scheme_validator: Optional[MaxRoasBiddingScheme] = None
    # data type: PercentCpcBiddingScheme
    oneof_schema_percent_cpc_bidding_scheme_validator: Optional[PercentCpcBiddingScheme] = None
    # data type: TargetCpaBiddingScheme
    oneof_schema_target_cpa_bidding_scheme_validator: Optional[TargetCpaBiddingScheme] = None
    # data type: TargetImpressionShareBiddingScheme
    oneof_schema_target_impression_share_bidding_scheme_validator: Optional[TargetImpressionShareBiddingScheme] = None
    # data type: TargetRoasBiddingScheme
    oneof_schema_target_roas_bidding_scheme_validator: Optional[TargetRoasBiddingScheme] = None
    actual_instance: Optional[Union[CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme]] = None
    one_of_schemas: Set[str] = { "CommissionBiddingScheme", "CostPerSaleBiddingScheme", "EnhancedCpcBiddingScheme", "InheritFromParentBiddingScheme", "ManualCpaBiddingScheme", "ManualCpcBiddingScheme", "ManualCpmBiddingScheme", "ManualCpvBiddingScheme", "MaxClicksBiddingScheme", "MaxConversionValueBiddingScheme", "MaxConversionsBiddingScheme", "MaxRoasBiddingScheme", "PercentCpcBiddingScheme", "TargetCpaBiddingScheme", "TargetImpressionShareBiddingScheme", "TargetRoasBiddingScheme" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = BiddingScheme.model_construct()
        error_messages = []
        match = 0
        # validate data type: CommissionBiddingScheme
        if not isinstance(v, CommissionBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CommissionBiddingScheme`")
        else:
            match += 1
        # validate data type: CostPerSaleBiddingScheme
        if not isinstance(v, CostPerSaleBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CostPerSaleBiddingScheme`")
        else:
            match += 1
        # validate data type: EnhancedCpcBiddingScheme
        if not isinstance(v, EnhancedCpcBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `EnhancedCpcBiddingScheme`")
        else:
            match += 1
        # validate data type: InheritFromParentBiddingScheme
        if not isinstance(v, InheritFromParentBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `InheritFromParentBiddingScheme`")
        else:
            match += 1
        # validate data type: ManualCpaBiddingScheme
        if not isinstance(v, ManualCpaBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ManualCpaBiddingScheme`")
        else:
            match += 1
        # validate data type: ManualCpcBiddingScheme
        if not isinstance(v, ManualCpcBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ManualCpcBiddingScheme`")
        else:
            match += 1
        # validate data type: ManualCpmBiddingScheme
        if not isinstance(v, ManualCpmBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ManualCpmBiddingScheme`")
        else:
            match += 1
        # validate data type: ManualCpvBiddingScheme
        if not isinstance(v, ManualCpvBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ManualCpvBiddingScheme`")
        else:
            match += 1
        # validate data type: MaxClicksBiddingScheme
        if not isinstance(v, MaxClicksBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `MaxClicksBiddingScheme`")
        else:
            match += 1
        # validate data type: MaxConversionValueBiddingScheme
        if not isinstance(v, MaxConversionValueBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `MaxConversionValueBiddingScheme`")
        else:
            match += 1
        # validate data type: MaxConversionsBiddingScheme
        if not isinstance(v, MaxConversionsBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `MaxConversionsBiddingScheme`")
        else:
            match += 1
        # validate data type: MaxRoasBiddingScheme
        if not isinstance(v, MaxRoasBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `MaxRoasBiddingScheme`")
        else:
            match += 1
        # validate data type: PercentCpcBiddingScheme
        if not isinstance(v, PercentCpcBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PercentCpcBiddingScheme`")
        else:
            match += 1
        # validate data type: TargetCpaBiddingScheme
        if not isinstance(v, TargetCpaBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TargetCpaBiddingScheme`")
        else:
            match += 1
        # validate data type: TargetImpressionShareBiddingScheme
        if not isinstance(v, TargetImpressionShareBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TargetImpressionShareBiddingScheme`")
        else:
            match += 1
        # validate data type: TargetRoasBiddingScheme
        if not isinstance(v, TargetRoasBiddingScheme):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TargetRoasBiddingScheme`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in BiddingScheme with oneOf schemas: CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in BiddingScheme with oneOf schemas: CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `CommissionBiddingScheme`
        if _data_type == "CommissionBiddingScheme":
            instance.actual_instance = CommissionBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `CostPerSaleBiddingScheme`
        if _data_type == "CostPerSale":
            instance.actual_instance = CostPerSaleBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `EnhancedCpcBiddingScheme`
        if _data_type == "EnhancedCpc":
            instance.actual_instance = EnhancedCpcBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `InheritFromParentBiddingScheme`
        if _data_type == "InheritFromParent":
            instance.actual_instance = InheritFromParentBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpaBiddingScheme`
        if _data_type == "ManualCpaBiddingScheme":
            instance.actual_instance = ManualCpaBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpcBiddingScheme`
        if _data_type == "ManualCpc":
            instance.actual_instance = ManualCpcBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpmBiddingScheme`
        if _data_type == "ManualCpm":
            instance.actual_instance = ManualCpmBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpvBiddingScheme`
        if _data_type == "ManualCpv":
            instance.actual_instance = ManualCpvBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxClicksBiddingScheme`
        if _data_type == "MaxClicks":
            instance.actual_instance = MaxClicksBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxConversionValueBiddingScheme`
        if _data_type == "MaxConversionValueBiddingScheme":
            instance.actual_instance = MaxConversionValueBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxConversionsBiddingScheme`
        if _data_type == "MaxConversions":
            instance.actual_instance = MaxConversionsBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxRoasBiddingScheme`
        if _data_type == "MaxRoasBiddingScheme":
            instance.actual_instance = MaxRoasBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `PercentCpcBiddingScheme`
        if _data_type == "PercentCpcBiddingScheme":
            instance.actual_instance = PercentCpcBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `TargetCpaBiddingScheme`
        if _data_type == "TargetCpa":
            instance.actual_instance = TargetCpaBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `TargetImpressionShareBiddingScheme`
        if _data_type == "TargetImpressionShareBiddingScheme":
            instance.actual_instance = TargetImpressionShareBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `TargetRoasBiddingScheme`
        if _data_type == "TargetRoasBiddingScheme":
            instance.actual_instance = TargetRoasBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `CostPerSaleBiddingScheme`
        if _data_type == "CostPerSaleBiddingScheme":
            instance.actual_instance = CostPerSaleBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `EnhancedCpcBiddingScheme`
        if _data_type == "EnhancedCpcBiddingScheme":
            instance.actual_instance = EnhancedCpcBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `InheritFromParentBiddingScheme`
        if _data_type == "InheritFromParentBiddingScheme":
            instance.actual_instance = InheritFromParentBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpcBiddingScheme`
        if _data_type == "ManualCpcBiddingScheme":
            instance.actual_instance = ManualCpcBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpmBiddingScheme`
        if _data_type == "ManualCpmBiddingScheme":
            instance.actual_instance = ManualCpmBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `ManualCpvBiddingScheme`
        if _data_type == "ManualCpvBiddingScheme":
            instance.actual_instance = ManualCpvBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxClicksBiddingScheme`
        if _data_type == "MaxClicksBiddingScheme":
            instance.actual_instance = MaxClicksBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `MaxConversionsBiddingScheme`
        if _data_type == "MaxConversionsBiddingScheme":
            instance.actual_instance = MaxConversionsBiddingScheme.from_json(json_str)
            return instance
			
		# check if data type is `TargetCpaBiddingScheme`
        if _data_type == "TargetCpaBiddingScheme":
            instance.actual_instance = TargetCpaBiddingScheme.from_json(json_str)
            return instance
			

        # deserialize data into CommissionBiddingScheme
        try:
            instance.actual_instance = CommissionBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CostPerSaleBiddingScheme
        try:
            instance.actual_instance = CostPerSaleBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into EnhancedCpcBiddingScheme
        try:
            instance.actual_instance = EnhancedCpcBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into InheritFromParentBiddingScheme
        try:
            instance.actual_instance = InheritFromParentBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ManualCpaBiddingScheme
        try:
            instance.actual_instance = ManualCpaBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ManualCpcBiddingScheme
        try:
            instance.actual_instance = ManualCpcBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ManualCpmBiddingScheme
        try:
            instance.actual_instance = ManualCpmBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ManualCpvBiddingScheme
        try:
            instance.actual_instance = ManualCpvBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into MaxClicksBiddingScheme
        try:
            instance.actual_instance = MaxClicksBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into MaxConversionValueBiddingScheme
        try:
            instance.actual_instance = MaxConversionValueBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into MaxConversionsBiddingScheme
        try:
            instance.actual_instance = MaxConversionsBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into MaxRoasBiddingScheme
        try:
            instance.actual_instance = MaxRoasBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PercentCpcBiddingScheme
        try:
            instance.actual_instance = PercentCpcBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into TargetCpaBiddingScheme
        try:
            instance.actual_instance = TargetCpaBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into TargetImpressionShareBiddingScheme
        try:
            instance.actual_instance = TargetImpressionShareBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into TargetRoasBiddingScheme
        try:
            instance.actual_instance = TargetRoasBiddingScheme.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into BiddingScheme with oneOf schemas: CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into BiddingScheme with oneOf schemas: CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], CommissionBiddingScheme, CostPerSaleBiddingScheme, EnhancedCpcBiddingScheme, InheritFromParentBiddingScheme, ManualCpaBiddingScheme, ManualCpcBiddingScheme, ManualCpmBiddingScheme, ManualCpvBiddingScheme, MaxClicksBiddingScheme, MaxConversionValueBiddingScheme, MaxConversionsBiddingScheme, MaxRoasBiddingScheme, PercentCpcBiddingScheme, TargetCpaBiddingScheme, TargetImpressionShareBiddingScheme, TargetRoasBiddingScheme]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_commission_bidding_scheme_validator', 'oneof_schema_cost_per_sale_bidding_scheme_validator', 'oneof_schema_enhanced_cpc_bidding_scheme_validator', 'oneof_schema_inherit_from_parent_bidding_scheme_validator', 'oneof_schema_manual_cpa_bidding_scheme_validator', 'oneof_schema_manual_cpc_bidding_scheme_validator', 'oneof_schema_manual_cpm_bidding_scheme_validator', 'oneof_schema_manual_cpv_bidding_scheme_validator', 'oneof_schema_max_clicks_bidding_scheme_validator', 'oneof_schema_max_conversion_value_bidding_scheme_validator', 'oneof_schema_max_conversions_bidding_scheme_validator', 'oneof_schema_max_roas_bidding_scheme_validator', 'oneof_schema_percent_cpc_bidding_scheme_validator', 'oneof_schema_target_cpa_bidding_scheme_validator', 'oneof_schema_target_impression_share_bidding_scheme_validator', 'oneof_schema_target_roas_bidding_scheme_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from typing_extensions import Self

class NegativeKeywordList(BaseModel):
    """
    NegativeKeywordList
    """ # noqa: E501
    item_count: Optional[StrictInt] = Field(default=None, alias="ItemCount")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    type: Optional[StrictStr] = Field(default='NegativeKeywordList', alias="Type")
    association_count: Optional[StrictInt] = Field(default=None, alias="AssociationCount")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["ItemCount", "Id", "Name", "Type", "AssociationCount", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NegativeKeywordList from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if item_count (nullable) is None
        # and model_fields_set contains the field
        if self.item_count is None and "item_count" in self.model_fields_set:
            _dict['ItemCount'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        # set to None if association_count (nullable) is None
        # and model_fields_set contains the field
        if self.association_count is None and "association_count" in self.model_fields_set:
            _dict['AssociationCount'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NegativeKeywordList from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "ItemCount": obj.get("ItemCount") if obj.get("ItemCount") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'NegativeKeywordList',
                        "AssociationCount": obj.get("AssociationCount") if obj.get("AssociationCount") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.address import Address
from openapi_client.models.customer.customer_financial_status import CustomerFinancialStatus
from openapi_client.models.customer.customer_life_cycle_status import CustomerLifeCycleStatus
from openapi_client.models.customer.industry import Industry
from openapi_client.models.customer.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.customer.language_type import LanguageType
from openapi_client.models.customer.service_level import ServiceLevel
from typing_extensions import Self

class Customer(BaseModel):
    """
    Customer
    """ # noqa: E501
    customer_financial_status: Optional[CustomerFinancialStatus] = Field(default=None, alias="CustomerFinancialStatus")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    industry: Optional[Industry] = Field(default=None, alias="Industry")
    last_modified_by_user_id: Optional[StrictStr] = Field(default=None, alias="LastModifiedByUserId")
    last_modified_time: Optional[datetime] = Field(default=None, alias="LastModifiedTime")
    market_country: Optional[StrictStr] = Field(default=None, alias="MarketCountry")
    market_language: Optional[LanguageType] = Field(default=None, alias="MarketLanguage")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    service_level: Optional[ServiceLevel] = Field(default=None, alias="ServiceLevel")
    customer_life_cycle_status: Optional[CustomerLifeCycleStatus] = Field(default=None, alias="CustomerLifeCycleStatus")
    time_stamp: Optional[StrictStr] = Field(default=None, alias="TimeStamp")
    number: Optional[StrictStr] = Field(default=None, alias="Number")
    customer_address: Optional[Address] = Field(default=None, alias="CustomerAddress")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["CustomerFinancialStatus", "Id", "Industry", "LastModifiedByUserId", "LastModifiedTime", "MarketCountry", "MarketLanguage", "Name", "ServiceLevel", "CustomerLifeCycleStatus", "TimeStamp", "Number", "CustomerAddress", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of customer_address
        if self.customer_address:
            _dict['CustomerAddress'] = self.customer_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if customer_financial_status (nullable) is None
        # and model_fields_set contains the field
        if self.customer_financial_status is None and "customer_financial_status" in self.model_fields_set:
            _dict['CustomerFinancialStatus'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if industry (nullable) is None
        # and model_fields_set contains the field
        if self.industry is None and "industry" in self.model_fields_set:
            _dict['Industry'] = None

        # set to None if last_modified_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_by_user_id is None and "last_modified_by_user_id" in self.model_fields_set:
            _dict['LastModifiedByUserId'] = None

        # set to None if last_modified_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_time is None and "last_modified_time" in self.model_fields_set:
            _dict['LastModifiedTime'] = None

        # set to None if market_country (nullable) is None
        # and model_fields_set contains the field
        if self.market_country is None and "market_country" in self.model_fields_set:
            _dict['MarketCountry'] = None

        # set to None if market_language (nullable) is None
        # and model_fields_set contains the field
        if self.market_language is None and "market_language" in self.model_fields_set:
            _dict['MarketLanguage'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if service_level (nullable) is None
        # and model_fields_set contains the field
        if self.service_level is None and "service_level" in self.model_fields_set:
            _dict['ServiceLevel'] = None

        # set to None if customer_life_cycle_status (nullable) is None
        # and model_fields_set contains the field
        if self.customer_life_cycle_status is None and "customer_life_cycle_status" in self.model_fields_set:
            _dict['CustomerLifeCycleStatus'] = None

        # set to None if time_stamp (nullable) is None
        # and model_fields_set contains the field
        if self.time_stamp is None and "time_stamp" in self.model_fields_set:
            _dict['TimeStamp'] = None

        # set to None if number (nullable) is None
        # and model_fields_set contains the field
        if self.number is None and "number" in self.model_fields_set:
            _dict['Number'] = None

        # set to None if customer_address (nullable) is None
        # and model_fields_set contains the field
        if self.customer_address is None and "customer_address" in self.model_fields_set:
            _dict['CustomerAddress'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Customer from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CustomerFinancialStatus": obj.get("CustomerFinancialStatus") if obj.get("CustomerFinancialStatus") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Industry": obj.get("Industry") if obj.get("Industry") is not None else None,
                        "LastModifiedByUserId": obj.get("LastModifiedByUserId") if obj.get("LastModifiedByUserId") is not None else None,
                        "LastModifiedTime": obj.get("LastModifiedTime") if obj.get("LastModifiedTime") is not None else None,
                        "MarketCountry": obj.get("MarketCountry") if obj.get("MarketCountry") is not None else None,
                        "MarketLanguage": obj.get("MarketLanguage") if obj.get("MarketLanguage") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "ServiceLevel": obj.get("ServiceLevel") if obj.get("ServiceLevel") is not None else None,
                        "CustomerLifeCycleStatus": obj.get("CustomerLifeCycleStatus") if obj.get("CustomerLifeCycleStatus") is not None else None,
                        "TimeStamp": obj.get("TimeStamp") if obj.get("TimeStamp") is not None else None,
                        "Number": obj.get("Number") if obj.get("Number") is not None else None,
                        "CustomerAddress": Address.from_dict(obj["CustomerAddress"]) if obj.get("CustomerAddress") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.reporting.model_date import ModelDate
from openapi_client.models.reporting.report_time_period import ReportTimePeriod
from openapi_client.models.reporting.report_time_zone import ReportTimeZone
from typing_extensions import Self

class ReportTime(BaseModel):
    """
    ReportTime
    """ # noqa: E501
    predefined_time: Optional[ReportTimePeriod] = Field(default=None, alias="PredefinedTime")
    custom_date_range_start: Optional[ModelDate] = Field(default=None, alias="CustomDateRangeStart")
    custom_date_range_end: Optional[ModelDate] = Field(default=None, alias="CustomDateRangeEnd")
    report_time_zone: Optional[ReportTimeZone] = Field(default=None, alias="ReportTimeZone")
    __properties: ClassVar[List[str]] = ["PredefinedTime", "CustomDateRangeStart", "CustomDateRangeEnd", "ReportTimeZone"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of custom_date_range_start
        if self.custom_date_range_start:
            _dict['CustomDateRangeStart'] = self.custom_date_range_start.to_dict()
        # override the default output from pydantic by calling `to_dict()` of custom_date_range_end
        if self.custom_date_range_end:
            _dict['CustomDateRangeEnd'] = self.custom_date_range_end.to_dict()
        # set to None if predefined_time (nullable) is None
        # and model_fields_set contains the field
        if self.predefined_time is None and "predefined_time" in self.model_fields_set:
            _dict['PredefinedTime'] = None

        # set to None if custom_date_range_start (nullable) is None
        # and model_fields_set contains the field
        if self.custom_date_range_start is None and "custom_date_range_start" in self.model_fields_set:
            _dict['CustomDateRangeStart'] = None

        # set to None if custom_date_range_end (nullable) is None
        # and model_fields_set contains the field
        if self.custom_date_range_end is None and "custom_date_range_end" in self.model_fields_set:
            _dict['CustomDateRangeEnd'] = None

        # set to None if report_time_zone (nullable) is None
        # and model_fields_set contains the field
        if self.report_time_zone is None and "report_time_zone" in self.model_fields_set:
            _dict['ReportTimeZone'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ReportTime from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "PredefinedTime": obj.get("PredefinedTime") if obj.get("PredefinedTime") is not None else None,
                        "CustomDateRangeStart": ModelDate.from_dict(obj["CustomDateRangeStart"]) if obj.get("CustomDateRangeStart") is not None else None,
                        "CustomDateRangeEnd": ModelDate.from_dict(obj["CustomDateRangeEnd"]) if obj.get("CustomDateRangeEnd") is not None else None,
                        "ReportTimeZone": obj.get("ReportTimeZone") if obj.get("ReportTimeZone") is not None else None
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ReportTimePeriod(str, Enum):
    TODAY = 'Today'

    YESTERDAY = 'Yesterday'

    LASTSEVENDAYS = 'LastSevenDays'

    THISWEEK = 'ThisWeek'

    LASTWEEK = 'LastWeek'

    LAST14DAYS = 'Last14Days'

    LAST30DAYS = 'Last30Days'

    LASTFOURWEEKS = 'LastFourWeeks'

    THISMONTH = 'ThisMonth'

    LASTMONTH = 'LastMonth'

    LASTTHREEMONTHS = 'LastThreeMonths'

    LASTSIXMONTHS = 'LastSixMonths'

    THISYEAR = 'ThisYear'

    LASTYEAR = 'LastYear'

    THISWEEKSTARTINGMONDAY = 'ThisWeekStartingMonday'

    LASTWEEKSTARTINGMONDAY = 'LastWeekStartingMonday'

    LASTFOURWEEKSSTARTINGMONDAY = 'LastFourWeeksStartingMonday'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReportTimePeriod from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkLabel(_SingleRecordBulkEntity):
    """ Represents a label that can be read or written in a bulk file.

    This class exposes the :attr:`label` property that can be read and written as fields of the Keyword record in a bulk file.
    Properties of this class and of classes that it is derived from, correspond to fields of the Keyword record in a bulk file.
    For more information, see Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label=None, status=None):
        super(BulkLabel, self).__init__()
        self._label = label
        self._status = status

    @property
    def label(self):
        """ The Label Data Object of the Campaign Management Service.

        A subset of Label properties are available in the Ad Group record.
        """

        return self._label

    @label.setter
    def label(self, value):
        self._label = value

    @property
    def status(self):
        """ the status of bulk record
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(
                c,
                'status',
                v if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: c.label.Id,
            csv_to_field=lambda c, v: setattr(
                c.label,
                'Id',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.ColorCode,
            field_to_csv=lambda c: c.label.ColorCode,
            csv_to_field=lambda c, v: setattr(
                c.label,
                'ColorCode',
                v
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Description,
            field_to_csv=lambda c: c.label.Description,
            csv_to_field=lambda c, v: setattr(c.label, 'Description', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Label,
            field_to_csv=lambda c: c.label.Name,
            csv_to_field=lambda c, v: setattr(c.label, 'Name', v)
        ),
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._label, 'label')
        self.convert_to_values(row_values, BulkLabel._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._label = _CAMPAIGN_OBJECT_FACTORY_V13.create('Label')
        row_values.convert_to_entity(self, BulkLabel._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkLabel, self).read_additional_data(stream_reader)
